<?php
/**
* Provides interface for making all administrative database changes
* @author Nick Korbel <lqqkout13@users.sourceforge.net>
* @version 12-14-03
* @package Admin
*/
require_once('lib/Admin.class.php');

// Make sure this is the admin and is being called from admin.php
if ( (!Auth::isAdmin()) || (!strstr($_SERVER['HTTP_REFERER'],'admin.php')) ) {
    CmnFns::do_error_box('This section is only available to the administrator.<br />'
        . '<a href="ctrlpnl.php">Back to My Control Panel</a>');
	die;
}


$db = new AdminDB();

$tools = array ( 
				'deleteUsers' => 'del_users',
				
				'addResource'	=> 'add_resource',
				'editResource'	=> 'edit_resource',
				'delResource'	=> 'del_resource',
				'togResource'	=> 'tog_resource',
				
				'editPerms' =>	'edit_perms'
				 );

$fn = isset($_POST['fn']) ? $_POST['fn'] : (isset($_GET['fn']) ? $_GET['fn'] : '');	// Set function

if (!isset($tools[$fn]) && !isset($tools[$fn])) {		// Validate tool
	CmnFns::do_error_box('Unknown administrative function.'
				 . '<a href="ctrlpnl.php">Back to My Control Panel</a>');
	die;
}
else {
	if (isset($tools[$fn]))
		eval($tools[$fn] . '();');
}

unset($fn, $tools);

/**
* Deletes a list of users from the database
* @param none
*/
function del_users() {
	global $db;
	
	// Make sure memberids are checked
	if (empty($_POST['memberid']))
		print_fail('You did not select any members to delete.<br />');
	
	$db->del_users($_POST['memberid']);
	CmnFns::write_log('Users deleted. ' . join(', ', $_POST['memberid']), $_SESSION['sessionID']);
	print_success();
}


/**
* Adds a resource to the database
* @param none
*/
function add_resource() {
	global $db;
	global $conf;
	
	$resource = check_resource_data(CmnFns::cleanPostVals());
	$id = $db->add_resource($resource);
	
	if ($conf['app']['auto_assign'])		// Automatically give all users permission to reserve this resource
		$db->auto_assign($id);
	
	CmnFns::write_log('Resource added. ' . $resource['name'], $_SESSION['sessionID']);
	print_success();
}

/**
* Edits resource data
* @param none
*/
function edit_resource() {
	global $db;
	
	$resource = check_resource_data(CmnFns::cleanPostVals());
	$db->edit_resource($resource);
	CmnFns::write_log('Resource editied. ' . $resource['name'] . ' ' . $resource['machid'], $_SESSION['sessionID']);
	print_success();
}

/**
* Deletes a list of resources
* @param none
*/
function del_resource() {
	global $db;
	
	// Make sure machids are checked
	if (empty($_POST['machid']))
		print_fail('You did not select any resources to delete.');
	
	$db->del_resource($_POST['machid']);
	CmnFns::write_log('Resources deleted. ' . join(', ', $_POST['machid']), $_SESSION['sessionID']);
	print_success();
}

/**
* Toggles a resource active/inactive
* @param none
*/
function tog_resource() {
	global $db;
	
	$db->tog_resource($_GET['machid'], $_GET['status']);
	CmnFns::write_log('Resource ' . $_GET['machid'] . ' toggled on/off.', $_SESSION['sessionID']);
	print_success();
}

/**
* Validates resource data
* @param array $data array of data to validate
* @return validated data
*/
function check_resource_data($data) {
	$rs = array();
	$msg = array();
	
	if (empty($data['name']))
		array_push($msg, 'Resource name is required.');
	else
		$rs['name'] = $data['name'];
	
	if (intval($data['minRes']) > intval($data['maxRes']))
		array_push($msg, 'Minimum reservation length must be less than or equal to maximum reservation length.');
	else {
		$rs['minRes']	= $data['minRes'];
		$rs['maxRes']	= $data['maxRes'];
	}
	
	$rs['rphone']	= $data['rphone'];
	$rs['location'] = $data['location'];
	$rs['notes']	= $data['notes'];

	
	if (isset($data['machid']))
		$rs['machid'] = $data['machid'];
	
	if (!empty($msg))
		print_fail($msg, $data);
	
	return $rs;

}

/**
* Edit user permissions for what resources they can reserve
* @param none
*/ 
function edit_perms() {
	global $db;
	
	$db->clear_perms($_POST['memberid']);
	$db->set_perms($_POST['memberid'], isset($_POST['machid']) ? $_POST['machid'] : array());
	CmnFns::write_log('Permissions changed for user ' . $_POST['memberid'], $_SESSION['sessionID']);
	print_success();
}

/**
* Prints a page with a message notifying the admin of a successful update
* @param none
*/
function print_success() {
	// Get the name/value of anything that was currently being edited
	// This will then be flitered out of the link back so that item will not show up in the edit box
	$return = (!empty($_POST['get'])) ? preg_replace('/&' . $_POST['get'] . '=[\d\w]*/', '', $_SERVER['HTTP_REFERER']) : $_SERVER['HTTP_REFERER'];
	header("Refresh: 2; URL=$return");		// Auto send back after 2 seconds
	$t = new Template('Successful update');
	$t->printHTMLHeader();
	$t->printWelcome();
	$t->startMain();
	CmnFns::do_message_box('Your request was processed successfully.<br />'
				. '<a href="' . $return. '">Go back to system administration</a><br />'
				. 'Or wait to be automatically redirected there.');
	$t->endMain();
	$t->printHTMLFooter();
	die;	
}


/**
* Prints a page notifiying the admin that the requirest failed.
* It will also assign the data passed in to a session variable
*  so it can be reinserted into the form that it came from
* @param string or array $msg message(s) to print to user
* @param array $data array of data to post back into the form
*/
function print_fail($msg, $data = null) {
	if (!is_array($msg))
		$msg = array ($msg);
		
	if (!empty($data))
		$_SESSION['post'] = $data;
	
	$t = new Template('Update failed!');
	$t->printHTMLHeader();
	$t->printWelcome();
	$t->startMain();
	CmnFns::do_error_box('There were problems processing your request.<br /><br />'
			. '- ' . join('<br />- ', $msg) . '<br />'
			. '<br />Please <a href="' . $_SERVER['HTTP_REFERER'] . '">go back</a> and correct any errors.');
	$t->endMain();
	$t->printHTMLFooter();
	die;
}
?>