<?

/**
	* Copyright (C) 2003 Tony Bierman
	* 
	* This program is free software; you can redistribute it and/or
	* modify it under the terms of the GNU General Public License
	* as published by the Free Software Foundation; either version 2
	* of the License, or (at your option) any later version.
	* This program is distributed in the hope that it will be useful,
	* but WITHOUT ANY WARRANTY; without even the implied warranty of
	* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	* GNU General Public License for more details.
	* You should have received a copy of the GNU General Public License
	* along with this program; if not, write to the Free Software
	* Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
	*
        *PURPOSE:
	* phpGiggle locates user-definable keywords in HTML string (or plain text for that matter),
        * and subsequently replaces the keywords with anchor tags of user-definable URLs.
        * As well, pre-defined 'template' anchor tags are available for Google.com searches or
        * Dictionary.com lookups.
	*
        * phpGiggle also takes care that no replacement inside an HTML tags is made, and also avoids matching
        * keywords which were found inside of a pre-existing anchor tag's <a>text</a>.
        * 
        *
        * function giggle_autolink() takes the following parameters:
        *
        * <$string> - HTML (or plain text) string to parse
        *
        * [$arr_keywords] - an optional parameter which can either take the form of an array
        *   in the format key => value or a $string in the form 'key = value'
        *   where key is the keyword to locate, and value can be one of the following values:
        *       1. URL in the form of e.g. 'http://www.biermana.org'
        *       2. literal text 'google'
        *       3. literal text 'dictionary'
        *       4. valid HTML hexadecimal identifier e.g. #FFFF00 or #000066 - in this case, the keyword will be highlighted
        *   If no value is specified for $arr_keywords, giggle_autolink will read from the phpgiggle.word.txt file (default behavior)
        *   
        *  [$b_noprint] - optional parameter defaults to 0, meaning giggle_autolink will echo modified
        *       HTML to browser.  If set to 1, giggle_autolink will return modified HTML to caller as string variable. 
	*
	* Changelog:
	* 	V1.0.1 Initial Version 01/25/2004
        * Author:
        * 	biermana@biemana.org - http://www.biermana.org
	*
        *INSTALLATION:
        *
        *These instructions will assume you've downloaded the source tarball and upacked to a world-readable folder
        *on your http server and all source files are located in <server root>/phpGiggle
        *
        *STEP 1.
        *Include the phpGiggle function file
        *
        <!-- giggle --> 
	<?php include ("phpGiggle/phpgiggle.inc.php"); ?>
        
        STEP 2.
        Call the giggle_autolink function
        
        * giggle_autolink will default to reading in the phpgiggle.words.txt file for keywords:
        *<?
        *	giggle_autolink($html_string);
        *?>
	*
        * or you may pass an anonymous array to giggle_autolink.  The array should be in the form of
        * $keyword_array["Yahoo"] = "http://www.yahoo.com":
        *<?
        *	giggle_autolink($html_string,$keyword_array);
        *?>
        *
        *or finally, use the 'key = value' notation to call giggle_autolink
        *
        *In the above $config->param is a key value array with
        *elements such as $config->param["Yahoo"] = "http://www.yahoo.com"
        *
	*Alternatively, giggle_autolink may also be called with a single " = " separated key value pair as thus: 
        *<?
        *	giggle_autolink($html_string,"Yahoo = http://www.yahoo.com");
        *?>
        *
        *The above call with create anchor tags to www.yahoo.com for all instances of Yahoo
        *
        *INSTALL FOR WORDPRESS
        *WordPress is a LAMP weblog located at http://wordpress.org
        *For more about the my-hacks.php, see http://wiki.wordpress.org/index.php/my-hacks.php
        *
        *There is an example WordPress my-hacks.php included with the phpGiggle tarball.
        *Copy phpGiggle/my-hacks.php to the your WordPress directory
        *or append to your existing my-hacks.php file.
        *
        *
        *
        *That should be all that is needed to get started.
        *
        *Some future releases of phpGiggle may include
        *1. web-based admin gui for config files
        *2. Color highlighting (like google) instead of anchoring keywords
        *3. Javascript of DHTML context menu for multiple options on a keyword
        *4. Read directly from HTML file
        *5. mySQL backend instead of keyword file
        *
        *DEMO:
        *See my home page at http://www.biermana.org for a working demo
        *
        *Have Fun!,
        *Tony Bierman
*/

// creates anchors based on config file key => values
// tried to make it smart, fast
include($_SERVER['DOCUMENT_ROOT'] . '/phpGiggle/configreader.class.php');

function giggle_autolink($string,$arr_keywords=-1,$b_noprint=0)
{

    if($arr_keywords == -1 )
    {
        $config = new Config($_SERVER['DOCUMENT_ROOT'] . '/phpGiggle/phpgiggle.words.txt');    
        if(is_array($config->param))
        {
            $arr_keywords = $config->param;   
        }
        else
        {
            return $string;
        }        
    }
    
    // allow single keyword
    if(!is_array($arr_keywords)) 
    {
        $match = preg_split('/ = /', $arr_keywords);
        unset($arr_keywords);
        $arr_keywords[$match[0]]=$match[1];
    } 
    
    // pre-procesor - get rid of unneeded key words
    // saves time for potentially large lists
    foreach($arr_keywords as $giggle_word => $giggle_link )
    {
        $giggle_word = str_replace("_"," ",$giggle_word);
        if(! preg_match("/(?:\b|\s|\"|\'|\,|\.(?<!\w))($giggle_word)(?:\s|\"|\'|\,|\.(?!\w)|(?:\&\#8217\;s)|(?:\'s)|(\-))/i", $string)) { unset($arr_keywords[$giggle_word]); }
    }
     
    if (is_array($arr_keywords)) { // if keyword array still has elements, matches have been found
        foreach($arr_keywords as $giggle_word => $giggle_link)  // for each keyword
        {					
            $giggle_word = str_replace("_"," ",$giggle_word);
            $anchor_flag=0;
            // matches all text between "> <" in HTML
            // with PREG_SPLIT_DELIM_CAPTURE, this regexp also captures URLs inside of anchor tags
            foreach(preg_split("/<(\"[^\"]*\"|\'[^\']*\'|[^\'\">])*>/", $string, -1, PREG_SPLIT_DELIM_CAPTURE) as $match) {
                if($anchor_flag) // if previous was anchor tag, don't process this match unless color highlight
                {
                    $anchor_flag=0;
                    if(!preg_match("/^#?([a-f]|[A-F]|[0-9]){3}(([a-f]|[A-F]|[0-9]){3})?$/", $giggle_link)) { continue; }
                } 
                if(preg_match("/^\"http/", $match)) { $anchor_flag=1; continue; } // if anchor tag, don't process this match
                if(preg_match("/(?:\b|\s|\"|\'|\,|\.(?<!\w))($giggle_word)(?:\s|\"|\'|\,|\.(?!\w)|(?:\&\#8217\;s)|(?:\'s)|(\-))/i", $match, $out))
                {
                    $case_sensitive_search_word = $out[1];
                    switch ($giggle_link) {
                        case 'google':
                            $newtext = str_replace($case_sensitive_search_word,"<a title=\"phpGiggle: Google.com search for '".
                                $case_sensitive_search_word ."' !\" href=\"".
                                "http://www.google.com/search?q=".$case_sensitive_search_word ."\">".
                                $case_sensitive_search_word."</a>", $match);
                            break;
                        case 'dictionary':
                            $newtext = str_replace($case_sensitive_search_word,"<a title=\"phpGiggle: Dictionary.com definition for '".
                                $case_sensitive_search_word ."' !\" href=\"".
                                "http://dictionary.reference.com/search?q=".$case_sensitive_search_word ."\">".
                                $case_sensitive_search_word."</a>", $match);
                            break;
                        case (preg_match("/^#?([a-f]|[A-F]|[0-9]){3}(([a-f]|[A-F]|[0-9]){3})?$/", $giggle_link)?$giggle_link:!$giggle_link):
                            $newtext = str_replace($case_sensitive_search_word,"<span title=\"phpGiggle Highlighter !\" style=\"background-color: ".
                                $giggle_link.";\">$case_sensitive_search_word</span>", $match);    
                            break;
                        default:
                            $newtext = str_replace($case_sensitive_search_word,"<a title=\"phpGiggle it!\" href=\"".
                                $giggle_link ."\">".$case_sensitive_search_word."</a>", $match);
                    }
                    $string = str_replace($match, $newtext, $string);
                } // if keyword found
            } // for each HTML match
        }  // for each keyword	
    } // if config has elements
    unset($arr_keywords);
    if($b_noprint)
    {
            return $string;
    }
    else
    {
            echo $string;
    }
}

function giggle_wordpress ($text)
{ 
    return giggle_autolink($text,-1,1);   
}
?>
