<?php
/**
* Interface form for placing/modifying/viewing a reservation
* This file will present a form for a user to
*  make a new reservation or modify/delete an old one.
* It will also allow other users to view this reservation.
* @author Nick Korbel <lqqkout13@users.sourceforge.net>
* @version 12-14-03
* @package phpScheduleIt
*/
/**
* Template class
*/
require_once('lib/Template.class.php');
/**
* Reservation class
*/
require_once('lib/Reservation.class.php');


// Make sure user is logged in
if (!Auth::is_logged_in()) {
    Auth::print_login_msg();
}

$t = new Template();

if (isset($_POST['submit']) && strstr($_SERVER['HTTP_REFERER'], $_SERVER['PHP_SELF'])) {
	$t->set_title('Processing Reservation');
	$t->printHTMLHeader();
	$t->startMain();

	process_reservation($_POST['fn']);
}
else {
	$res_info = getResInfo();
	$t->set_title($res_info['title']);
	$t->printHTMLHeader();
	$t->startMain();

	present_reservation($res_info['resid']);
}

// End main table
$t->endMain();

// Print HTML footer
$t->printHTMLFooter();

/**
* Processes a reservation request (add/del/edit)
* @param string $fn function to perform
*/
function process_reservation($fn) {
	$success = false;

	if (isset($_POST['resid']))
		$res = new Reservation($_POST['resid']);
	else if (isset($_GET['resid']))
		$res = new Reservation($_GET['resid']);
	else {
		$res = new Reservation();
		if (isset($_POST['repeat_day'])) {		// Check for reservation repeation
			$res->is_repeat = true;
			$repeat = get_repeat_dates($_POST['ts'], $_POST['repeat_day'], $_POST['duration']);
		}
		else
			$repeat = array($_POST['ts']);
	}

	if ($fn == 'create')
		$res->add_res($_POST['machid'], $_SESSION['sessionID'], $_POST['startTime'], $_POST['endTime'], $repeat, $_POST['minRes'], $_POST['maxRes']);
	else if ($fn == 'modify')
		$res->mod_res($_POST['startTime'], $_POST['endTime'], isset($_POST['del']), $_POST['minRes'], $_POST['maxRes'], isset($_POST['mod_recur']));
	else if ($fn == 'delete')
		$res->del_res(isset($_POST['mod_recur']));
}

/**
* Prints out reservation info depending on what parameters
*  were passed in through the query string
* @param none
*/
function present_reservation($resid) {
	// Get info about this reservation
	$res = new Reservation($resid);
	$res->print_res();
}


/**
* Return array of data from query string about this reservation
*  or about a new reservation being created
* @param none
*/
function getResInfo() {
	$res_info = array();

	// Determine title and set needed variables
	$res_info['type'] = $_GET['type'];
	switch($res_info['type']) {
		case 'r' :
			$res_info['title'] = 'New Reservation';
			$res_info['resid']	= null;
			break;
		case 'm' :
			$res_info['title'] = 'Modify Reservation';
			$res_info['resid'] = $_GET['resid'];
			break;
		case 'd' :
			$res_info['title'] = 'Delete Reservation';
			$res_info['resid'] = $_GET['resid'];
			break;
		default : $res_info['title'] = 'View Reservation';
			$res_info['resid'] = $_GET['resid'];
			break;
	}

	return $res_info;
}

/**
* Returns an array of all timestamps for repeat reservations
* @param int $res_date initial reservation date
* @param array $days_to_repeat days of week to repeat reservation
* @param int $duration weeks to repeat reservation for
* @return array of timestamps of reservations
*/
function get_repeat_dates($res_date, $days_to_repeat, $duration) {
	$day_of_week = date('w', $res_date);		// Day of week for initial reservation

	$res_dates = array($res_date);
	$date_vals = getdate($res_date);

	// Repeat for all weeks
	$additional_days = 0;
	for ($repeat_week = 0; $repeat_week < $duration; $repeat_week++) {		// Repeat for duration
		for ($i = 0; $i < count($days_to_repeat); $i++) {					// Repeat for all days selected
			$days_between = ($days_to_repeat[$i] - $day_of_week) + $additional_days;
			// If the day of week is less than reservation day of week, move ahead one week
			if ($days_to_repeat[$i] <= $day_of_week) {
				$days_between += 7;
			}
			$res_date = mktime(0,0,0, $date_vals['mon'], $date_vals['mday'] + $days_between);
			array_push($res_dates, $res_date);
		}
		$additional_days += 7;	// Move ahead one week
	}

	return $res_dates;
}
?>